use crate::api::{get_dict_result, get_jyutping_result, get_moedict_index, set_json_result};
use crate::formatter::{
    gen_dict_json_str, gen_dict_result_str, gen_jyutping_str, gen_translation_str,
    get_terminal_size,
};
use anyhow::anyhow;
use anyhow::Result;
use rand::prelude::{IteratorRandom, SliceRandom};
use reqwest::Client;

pub struct MeowdictRequest {
    pub client: Client,
    pub no_color: bool,
}

impl MeowdictRequest {
    pub async fn search_word_to_dict_result(
        &self,
        words: &[String],
        result_t2s: bool,
    ) -> Result<()> {
        let terminal_size = get_terminal_size();
        let meowdict_results = get_dict_result(&self.client, words).await?;
        let result =
            gen_dict_result_str(meowdict_results, terminal_size, self.no_color, result_t2s);
        println!("{}", result);

        Ok(())
    }

    pub async fn search_word_to_translation_result(
        &self,
        words: &[String],
        result_t2s: bool,
    ) -> Result<()> {
        let meowdict_results = get_dict_result(&self.client, words).await?;
        let result = gen_translation_str(meowdict_results, self.no_color, result_t2s);
        println!("{}", result);

        Ok(())
    }

    pub async fn search_word_to_jyutping_result(
        &self,
        words: &[String],
        result_t2s: bool,
    ) -> Result<()> {
        let jyutping_results = get_jyutping_result(&self.client, words).await?;
        let result = gen_jyutping_str(jyutping_results, self.no_color, result_t2s);
        println!("{}", result);

        Ok(())
    }

    pub async fn search_word_to_json_result(
        &self,
        words: Vec<String>,
        result_t2s: bool,
    ) -> Result<()> {
        let json_obj = set_json_result(&self.client, words).await;
        println!("{}", gen_dict_json_str(json_obj, result_t2s)?);

        Ok(())
    }

    pub async fn random_moedict_item(
        &self,
        result_t2s: bool,
        words: Option<Vec<String>>,
    ) -> Result<()> {
        let moedict_index = get_moedict_index(&self.client).await?;
        let rng = &mut rand::thread_rng();
        let terminal_size = get_terminal_size();
        let rand_words = match words {
            Some(words) => {
                let mut result = Vec::new();
                for word in words {
                    result.push(
                        moedict_index
                            .iter()
                            .filter(|x| x.contains(&word))
                            .choose(rng)
                            .ok_or_else(|| anyhow!("Cannot choose one!"))?
                            .to_owned(),
                    )
                }

                result
            }
            None => {
                vec![moedict_index
                    .choose(rng)
                    .ok_or_else(|| anyhow!("Cannot choose one!"))?
                    .to_owned()]
            }
        };
        let moedict_results = get_dict_result(&self.client, &rand_words).await?;
        let result = gen_dict_result_str(moedict_results, terminal_size, self.no_color, result_t2s);
        println!("{}", result);

        Ok(())
    }
}
